#include <stdio.h>
#define  MSB    0x80000000
#define  DCTBLOCK int*

class Component_spec_class{
public:
    unsigned int C,  //Component id
                 H,  //Horizontal sampling factor
                 V,  //Vertical  ....
                 Tq; //Quantization table destination selector
};

class Frame_header_class{
public:
    unsigned int SOF,  //Start of frame in different type
                 Lf,   //Length
                 P,    //Sample Precision (from the orignal image)
                 Y,    //Number of lines
                 X,    //Number of samples per line
                 Nf;   //Number of component in the frame
    Component_spec_class* Comp;  //Components  C H V Tq

    int get(FILE* in, unsigned int sof);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Scan_component_spec_class{
public:
    unsigned int Cs,   //Scan component selector
                 Td,   //DC table selector
                 Ta;   //AC table selector
};

class Scan_header_class{
public:
    unsigned int Ls,  //length
                 Ns,  //Number of components in the scan
                 Ss,  //Start of spectral or predictor selection
                 Se,  //End of spectral selection
                 Ah,  
                 Al;
    Scan_component_spec_class* Comp; //Components Cs Td Ta
                                     // from [0] to [Ns-1]
    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Quantization_table_class{
public:
    unsigned int Lq,    //length
                 Pq[4], //Quantization precision 8 or 16
                 Tq[4]; //1: this table is presented
    int          Q[4][64]; //Tables

    Quantization_table_class(void){
        Tq[0]=0; Tq[1]=0; Tq[2]=0; Tq[3]=0;
    }

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Huffman_table_class{
public:
    unsigned int Lh,    //Length
                 Tc[4][2], //1: this table is presented
                 Th[4], //1: this table is presented
                 L[4][2][16],
                 V[4][2][16][200]; //tables

    Huffman_table_class(void){
        Tc[0][0]=0; Tc[1][0]=0; Tc[2][0]=0; Tc[3][0]=0;
        Tc[0][1]=0; Tc[1][1]=0; Tc[2][1]=0; Tc[3][1]=0;
        Th[0]=0; Th[1]=0; Th[2]=0; Th[3]=0;
    }

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Restart_interval_class{
public:
    unsigned int Lr,Ri;

    Restart_interval_class(void){
        Ri=0;
    }

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Define_number_class{
public:
    unsigned int Ld,NL;

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Comment_class{
public:
    unsigned int Lc;
    char *Cm;

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Application_class{
public:
    unsigned int Lp;

    int get(FILE* in);
    //get data from file stream in
    //return 0 : correct       otherwise : error

    void text_show(void);
    //show data in text mode
};

class Parser_class{
    Frame_header_class  Frame_header;
    Scan_header_class   Scan_header;
    Define_number_class NL;
    Quantization_table_class QT;
    Huffman_table_class      HT;
    Restart_interval_class   RI;
    Comment_class            COM;
    Application_class        APP;

public:

//the following variables are for speed up.
    int nComp;              //number of Components in a scan
    int* qTab[10];          //quantization table for the i-th Comp in a scan
    unsigned int* dcTab[10];//dc HuffTab for the i-th Comp in a scan
    unsigned int* acTab[10];//ac HuffTab for the i-th Comp in a scan
    int nBlock[10];         //number of blocks in the i-th Comp in a scan
                            //                  i=0, ... ,Ns-1
    unsigned int YH,YV,Xsize,Ysize;

    void decode(FILE* in);

private:
    unsigned int
       decode_MCU(FILE* in, int* PDC, unsigned int& temp, int& index);

    void EnhanceQuantizationTable( int *qtab );
    void ScaleIDCT(int* matrix);
    void output(void);

};
